//
//  MapView.m
//  SubnetMapper
//
//  Created by Thomas Thiel on 25.06.08.
//  Copyright 2008 Thomas Thiel. All rights reserved.
//
// THIS CODE REPRESENTS FREE SOFTWARE AND IS RELEASED 
// UNDER THE GPL LICENSE VERSION 2. PLEASE RESPECT
// THE TERMS OF THIS LICENSE, WHICH CAN BE REVIEWED AT:
// http://www.gnu.org/licenses/gpl-2.0.html

#include <netdb.h>
#import "MyDocument.h"
#import "SubnetRect.h"
#import "MapView.h"

@implementation MapView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
			cachedRectangles = [[NSMutableArray arrayWithCapacity:100] retain ];
			cachedLines = [[NSMutableArray arrayWithCapacity:100] retain ];
			cachedNets = [[NSMutableArray arrayWithCapacity:100] retain ];
			[self setSelectedSubnet:nil];
			[self refreshUserDefaults];
			notesIcon = [[NSImage imageNamed:@"NSInfo"] retain ];
			[notesIcon setFlipped:YES];
			[notesIcon setSize:NSMakeSize(12,12)];
			pinIcon = [[NSImage imageNamed:@"pin.png"] retain ];
			[pinIcon setFlipped:YES];
			[pinIcon setSize:NSMakeSize(20,35)];
			[self setFindHostList:[NSArray array]];
			showHosts=NO;			
    }
    return self;
};

- (void)refreshUserDefaults {
	
	NSUserDefaults *userdefaults = [NSUserDefaults standardUserDefaults];
	
	page_x_offset = [userdefaults floatForKey:@"page_x_offset"];
	page_y_offset = [userdefaults floatForKey:@"page_y_offset"];
	ip_multiplier = [userdefaults floatForKey:@"ip_multiplier"];
	block_y_offset = [userdefaults floatForKey:@"block_y_offset"];
	row_height = [userdefaults floatForKey:@"row_height"];
	
	if (page_x_offset==0) {
		page_x_offset = 100.0;
		[userdefaults setFloat:page_x_offset forKey:@"page_x_offset"];
	};
	
	if (page_y_offset==0) {
		page_y_offset = 30;
		[userdefaults setFloat:page_y_offset forKey:@"page_y_offset"];
	};
	
	if (ip_multiplier==0) {
		ip_multiplier = 5.5;
		[userdefaults setFloat:ip_multiplier forKey:@"ip_multiplier"];
	};
	
	if (block_y_offset==0) {
		block_y_offset = 50.0;
		[userdefaults setFloat:block_y_offset forKey:@"block_y_offset"];
	};
	
	if (row_height==0) {
		row_height = 15.0;
		[userdefaults setFloat:row_height forKey:@"row_height"];
	};
	
	if ([userdefaults synchronize]==NO)	[(MyDocument*)_document displayError:@"Could not save data for user defaults in function refreshUSerDefaults." info:@"This will not affect the way SubnetMapper works in any way, except for the fact that all User preferences will be reset to defaults during the next run of this program. There is no direct solution for this problem as this function is part of the Cocoa Framework. You should not see this error message ever."];
};

- (void)drawRect:(NSRect)rect {

	[self setFrameSize:NSMakeSize([self frame].size.width,((block_y_offset + (((cachedNumberOfRows*2)+8)*row_height))+((cachedNumberOfSubnets+12)*row_height)))];
	
	NSMutableArray *selRects = [[NSMutableArray array] retain]; 
	
	// prepare Textstyles...
	NSParagraphStyle *stdStyle = [NSParagraphStyle defaultParagraphStyle]; 
	NSMutableParagraphStyle *rightAlignStyle = [stdStyle mutableCopyWithZone:NULL];
	[rightAlignStyle setAlignment:NSRightTextAlignment]; 

	NSDictionary *rightAlignStyleDictionary =
	[NSDictionary dictionaryWithObjectsAndKeys:rightAlignStyle,NSParagraphStyleAttributeName,[NSColor blackColor],NSForegroundColorAttributeName,NULL];

	NSMutableParagraphStyle *tabStyle = [stdStyle mutableCopyWithZone:NULL];	
	NSTextTab *tab_0 = [[[NSTextTab alloc] initWithType:NSLeftTabStopType location:130.0] autorelease];
	NSTextTab *tab_1 = [[[NSTextTab alloc] initWithType:NSCenterTabStopType location:265.0] autorelease];
	NSTextTab *tab_2 = [[[NSTextTab alloc] initWithType:NSLeftTabStopType location:300.0] autorelease];
	NSTextTab *tab_3 = [[[NSTextTab alloc] initWithType:NSLeftTabStopType location:410.0] autorelease];
	
	NSArray *tabArray = [NSArray arrayWithObjects:tab_0,tab_1,tab_2,tab_3,nil];

	[tabStyle setTabStops:tabArray];
	
	NSDictionary *tabStyleDictionaryBlack =
	[NSDictionary dictionaryWithObjectsAndKeys:tabStyle,NSParagraphStyleAttributeName,[NSColor blackColor],NSForegroundColorAttributeName,NULL];

	NSDictionary *tabStyleDictionaryRed =
	[NSDictionary dictionaryWithObjectsAndKeys:tabStyle,NSParagraphStyleAttributeName,[NSColor redColor],NSForegroundColorAttributeName,NULL];

	// erase the background by drawing white
	[[NSColor whiteColor] set];
	[NSBezierPath fillRect:rect];

	MyDocument* mydoc = (MyDocument*)_document;
	unsigned int i;

	// we draw the Cache of Markings etc.:

	// NOW: Markings and tabellary lines etc.

	// we use hardcoded black sweetness for all the lines and legends etc.
	[[NSColor blackColor] set];
	
	// we use ONE Bezier Path for everything. I am cheap.
	NSBezierPath* aPath = [NSBezierPath bezierPath];

	// draw round edges ans with a thin line.
	[aPath setLineCapStyle:NSRoundLineCapStyle];
	[aPath setLineWidth:1.0];
	
	// horizontal lines for table headers:
	// UPPER
	[aPath moveToPoint:NSMakePoint(page_x_offset-10, page_y_offset+row_height-2)];
	[aPath lineToPoint:NSMakePoint((page_x_offset-10)+(128.0*ip_multiplier+20), page_y_offset+row_height-2)];
	// LOWER
	[aPath moveToPoint:NSMakePoint(page_x_offset-10, (page_y_offset+ block_y_offset-2 + (row_height*(cachedNumberOfRows))))];
	[aPath lineToPoint:NSMakePoint((page_x_offset-10)+(128.0*ip_multiplier+20), (page_y_offset + block_y_offset-2)+ (row_height*(cachedNumberOfRows)))];
	// Draw 'em
	[aPath stroke];

	// reinitialize the path
	aPath = [NSBezierPath bezierPath];
	
	// do it nice again
	[aPath setLineCapStyle:NSRoundLineCapStyle];
	[aPath setLineWidth:1.0];
		
	// vertical lines at ip...
	// ...16
	[aPath moveToPoint:NSMakePoint(page_x_offset+(16*ip_multiplier), page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(16*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// (Legend /28)
	[aPath moveToPoint:NSMakePoint(page_x_offset+(1*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(5*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+3)))];
	// ...48
	[aPath moveToPoint:NSMakePoint(page_x_offset+(48*ip_multiplier), page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(48*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...80
	[aPath moveToPoint:NSMakePoint(page_x_offset+(80*ip_multiplier), page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(80*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...112
	[aPath moveToPoint:NSMakePoint(page_x_offset+(112*ip_multiplier), page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(112*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...32 (left one of the double line)
	[aPath moveToPoint:NSMakePoint(page_x_offset+(32*ip_multiplier)-1.3, page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(32*ip_multiplier)-1.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...32 (right one of the double line)
	[aPath moveToPoint:NSMakePoint(page_x_offset+(32*ip_multiplier)+1.3, page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(32*ip_multiplier)+1.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...96 (left one of the double line)
	[aPath moveToPoint:NSMakePoint(page_x_offset+(96*ip_multiplier)-1.3, page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(96*ip_multiplier)-1.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...96 (right one of the double line)
	[aPath moveToPoint:NSMakePoint(page_x_offset+(96*ip_multiplier)+1.3, page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(96*ip_multiplier)+1.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// Legend (/27) left
	[aPath moveToPoint:NSMakePoint(page_x_offset+(50*ip_multiplier)-2.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(54*ip_multiplier)-2.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+3)))];
	// Legend (/27) right
	[aPath moveToPoint:NSMakePoint(page_x_offset+(50*ip_multiplier)+2.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(54*ip_multiplier)+2.3, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+3)))];
	// draw 'em
	[aPath stroke];

	// Set up a line dash pattern.
	float lineDash[2];
	lineDash[0] = 7.0;
	lineDash[1] = 3.0;
	
	// reinitialize Bezier Path
	aPath = [NSBezierPath bezierPath];
	
	// a nice clear line with a line dashing pattern, please.
	[aPath setLineCapStyle:NSRoundLineCapStyle];
	[aPath setLineWidth:1.0];
	[aPath setLineDash:lineDash count:2 phase:0.0];	

	// dashed line at ip...
	// ...1
	[aPath moveToPoint:NSMakePoint(page_x_offset, page_y_offset)];
	[aPath lineToPoint:NSMakePoint(page_x_offset, page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...64
	[aPath moveToPoint:NSMakePoint(page_x_offset+(64*ip_multiplier), page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(64*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// ...128
	[aPath moveToPoint:NSMakePoint(page_x_offset+(128*ip_multiplier), page_y_offset )];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(128*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+1)))];
	// Legend (/26)
	[aPath moveToPoint:NSMakePoint(page_x_offset+(99*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))];
	[aPath lineToPoint:NSMakePoint(page_x_offset+(103*ip_multiplier), page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+3)))];

	// draw 'em
	[aPath stroke];
	
	// time to draw the legend into the table headers.
	NSString* aString;
	
	// counter  - dah - surprised? ;)
	unsigned int counter_legend=0;

	// iterate, generate and draw legend.
	for (counter_legend=0;counter_legend<=127;counter_legend+=8) {
		aString = [NSString stringWithFormat:@"%d",counter_legend];
		[aString drawAtPoint:NSMakePoint(page_x_offset+(counter_legend*ip_multiplier)+3.0, page_y_offset-2.0) withAttributes:nil];
	};

	// the same for 128-254
	for (counter_legend=128;counter_legend<=255;counter_legend+=8) {
		aString = [NSString stringWithFormat:@"%d",counter_legend];
		[aString drawAtPoint:NSMakePoint(page_x_offset+((counter_legend-128)*ip_multiplier)+3.0, (page_y_offset+ block_y_offset-2 + (row_height*(cachedNumberOfRows-1)))) withAttributes:nil];
	};

	// 255 feels special today. Place it away from the masses.
	aString = [NSString stringWithFormat:@"%d",255];
	[aString drawAtPoint:NSMakePoint(page_x_offset+(128*ip_multiplier)+3.0, (page_y_offset+ block_y_offset-2 + (row_height*(cachedNumberOfRows-1)))) withAttributes:nil];

	// The text part of the legend.
	aString = [NSString stringWithFormat:@"/28 NM: .240 WC: .15"];
	[aString drawAtPoint:NSMakePoint(page_x_offset+(6*ip_multiplier), ( page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))) withAttributes:nil];
	aString = [NSString stringWithFormat:@"/27 NM: .224 WC: .31"];
	[aString drawAtPoint:NSMakePoint(page_x_offset+(55*ip_multiplier), ( page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))) withAttributes:nil];
	aString = [NSString stringWithFormat:@"/26 NM: .192 WC: .63"];
	[aString drawAtPoint:NSMakePoint(page_x_offset+(104*ip_multiplier), ( page_y_offset + block_y_offset + (row_height*((cachedNumberOfRows*2)+2)))) withAttributes:nil];
	
	
	// now we draw the cache of subnet rectangles
  unsigned int count = [cachedRectangles count];
			
	// iterate the cache
	for (i=0;i<count;i++) {
		
		//transfer the cached data into NSRect structures
		NSRect tempRect;
		tempRect.origin=NSMakePoint([[[cachedRectangles objectAtIndex:i] getX] floatValue],[[[cachedRectangles objectAtIndex:i] getY] floatValue]);
		tempRect.size.width=[[[cachedRectangles objectAtIndex:i] getWidth] floatValue];
		tempRect.size.height=[[[cachedRectangles objectAtIndex:i] getHeight] floatValue];

		// choose user defined color.
		NSColor* rectColor = [[[cachedRectangles objectAtIndex:i] getSubnet] color];
		[rectColor set];
		
		// draw the background of the subnet rectangle
		[NSBezierPath fillRect:tempRect];
	
		// if the subnet is selected, mark its border in another color than black and make the line thicker.
		if ([[cachedRectangles objectAtIndex:i] isSelected]) {
			
			[selRects addObject:[cachedRectangles objectAtIndex:i]];
			
		} else {
			
			[[NSColor blackColor] set];
			[NSBezierPath setDefaultLineWidth:1.0];
			[NSBezierPath strokeRect:tempRect];
			
		};
		
		// draw the notesIcon
		if (([[[cachedRectangles objectAtIndex:i] getSubnet] notes]!=nil)&
				(![[[[[cachedRectangles objectAtIndex:i] getSubnet] notes] string] isEqualToString:@""]))
				[notesIcon drawAtPoint:NSMakePoint(tempRect.origin.x+tempRect.size.width-14.0,tempRect.origin.y+1) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:0.8];

		// prepare inverted rectangle color for string drawing
		NSColor* textColor = [NSColor colorWithDeviceRed:1.0-[[rectColor colorUsingColorSpace:[NSColorSpace genericRGBColorSpace]] redComponent] 
																							 green:1.0-[[rectColor colorUsingColorSpace:[NSColorSpace genericRGBColorSpace]] greenComponent] 
																								blue:1.0-[[rectColor colorUsingColorSpace:[NSColorSpace genericRGBColorSpace]] blueComponent] 
																							 alpha:[[rectColor colorUsingColorSpace:[NSColorSpace genericRGBColorSpace]] alphaComponent]];
		NSDictionary *rectStyleDictionary =
		[NSDictionary dictionaryWithObject:textColor
																forKey:NSForegroundColorAttributeName];
		
		
		tempRect.origin.x += 3.0;
		tempRect.size.width -= 6.0;
	
		// draw the Identifier into the subnet rectangle
		[[[[cachedRectangles objectAtIndex:i] getSubnet] getIdentifier] drawInRect:tempRect withAttributes:rectStyleDictionary];

	};

	// iterate all nets and write the net legend on the left side of the table
	// for (i=0;i<cachedNumberOfRows;i++) {
	for (i=0;i<[cachedNets count];i++) {
		NSRect tempRect;
		[[NSColor blackColor] set];
		tempRect.origin=NSMakePoint(0,page_y_offset + ((i+1) * row_height));
		tempRect.size.width=page_x_offset-10.0;
		tempRect.size.height=row_height;
		[(NSString*)[cachedNets objectAtIndex:i] drawInRect:tempRect withAttributes:rightAlignStyleDictionary];
		tempRect.origin=NSMakePoint(0,page_y_offset + block_y_offset + (cachedNumberOfRows*row_height)+((i) * row_height));
		[(NSString*)[cachedNets objectAtIndex:i] drawInRect:tempRect withAttributes:rightAlignStyleDictionary];
	};		
		
	if ( ![NSGraphicsContext currentContextDrawingToScreen] ) {
	
		aString = [NSString stringWithFormat:@"Identifier \tIP/CIDR \tSize \tBroadcast \tDescription"];
		[aString drawAtPoint:NSMakePoint(20,(page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+5)*row_height))-6.0) withAttributes:tabStyleDictionaryBlack];

		// reinitialize Bezier Path
		aPath = [NSBezierPath bezierPath];

		// a nice clear line, please
		[aPath setLineCapStyle:NSRoundLineCapStyle];
		[aPath setLineWidth:1.0];

		// net listing header line
		[aPath moveToPoint:NSMakePoint(10, (page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+5)*row_height))+12.0)];
		[aPath lineToPoint:NSMakePoint(828, (page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+5)*row_height))+12.0)];

		// draw it

		[aPath stroke];

		for (i=0;i<cachedNumberOfSubnets;i++) {

			Subnet *selSubnet = [mydoc getSubnetAtIndex:i]; 
			
			if ([selSubnet isSelected]) 
				[[selSubnet toString] drawAtPoint:NSMakePoint(20,page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+6)*row_height) + (i * row_height)) withAttributes:tabStyleDictionaryRed];
			else
				[[selSubnet toString] drawAtPoint:NSMakePoint(20,page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+6)*row_height) + (i * row_height)) withAttributes:tabStyleDictionaryBlack];
				
		};		

	};
	
	for (i=0;i<[selRects count];i++)
	{
		aPath = [NSBezierPath bezierPath];

		NSRect tempRect;
		tempRect.origin=NSMakePoint([[[selRects objectAtIndex:i] getX] floatValue],[[[selRects objectAtIndex:i] getY] floatValue]);
		tempRect.size.width=[[[selRects objectAtIndex:i] getWidth] floatValue];
		tempRect.size.height=[[[selRects objectAtIndex:i] getHeight] floatValue];
		
//		[[NSColor selectedTextBackgroundColor] set];
		[[NSColor redColor] set];
		[NSBezierPath setDefaultLineWidth:4.0];
		[NSBezierPath strokeRect:tempRect];

		[[NSColor blackColor] set];
		[NSBezierPath setDefaultLineWidth:1.0];
		[NSBezierPath strokeRect:tempRect];
		
	};
	
	if (showHosts) {
		
		NSArray *listOfHosts = [self getFindHostList];
		BOOL foundAHost=NO;

		for (i=0;i<[listOfHosts count];i++) {
			unsigned int momIP = [((NSNumber*)[listOfHosts objectAtIndex:i]) unsignedLongValue];
			int j;
			int row=0;
			unsigned long int lastnet=0;
			unsigned long int cidr24=((~0)<<8);

			
			for (j=0;j<cachedNumberOfSubnets;j++) {
				Subnet *momSubnet=[mydoc getSubnetAtIndex:j];

				if (lastnet!=([momSubnet getIP]&cidr24)) {
					lastnet=([momSubnet getIP]&cidr24);
					row+=[momSubnet getNumberOfCIDR24Blocks];
				};
				
				if ([momSubnet containsHost:momIP]) {
					foundAHost=YES;
					unsigned char lastchar = *((unsigned char*)((void*)&momIP));
          
					int k;
					unsigned int offset=0;
					unsigned long int offset_helper=[momSubnet getIP]&cidr24;
					
					for (k=0;k<[momSubnet getNumberOfCIDR24Blocks];k++)
					{
						if (!((momIP&cidr24)==offset_helper)) { 
							offset++;
							offset_helper=([momSubnet getIP]+((k+1)*256))&cidr24;
						}
						else k=[momSubnet getNumberOfCIDR24Blocks];
					};
					
					
					if (lastchar<128) {
						[pinIcon drawAtPoint:NSMakePoint((page_x_offset+(ip_multiplier*lastchar)),(page_y_offset+((row-[momSubnet getNumberOfCIDR24Blocks])+offset)*row_height)-5) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1];
					} else {
						[pinIcon drawAtPoint:NSMakePoint((page_x_offset+(ip_multiplier*(lastchar-128))),((page_y_offset+((row-[momSubnet getNumberOfCIDR24Blocks])+offset)*row_height)-5)+block_y_offset+((cachedNumberOfRows-1)*row_height)) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1];
					};

					
				};
				
			};
			
		};

		if (!foundAHost) {
			[mydoc displayError:@"None of your searched Hosts have been found." info:@"Maybe you did misspell a DNS Name. Or you just entered a wrong address."];
			showHosts=NO;
			[self setFindHostList:[NSArray array]];
		};

	};
		
	[_subnet_table reloadData];
	
	[selRects release];
			
};

- (BOOL)isFlipped {
	
	return YES;
	
}

- (void)refreshCache {
	
	// Beware!! Here be more dragons!
		
	// clear existing caches.
	[cachedRectangles removeAllObjects];
	[cachedLines removeAllObjects];
	[cachedNets removeAllObjects];

	
	// got some syntax errors if getting this link through direct casting, so i used this
	// workaround to shut up the compiler. he'll optimize this away in the resulting code anyway...
	MyDocument* mydoc = (MyDocument*)_document;
	
	// store these values locally so we don't have to use the ObjC messaging stuff anytime we need this data.
  cachedNumberOfRows = [mydoc getNumberOfNets];
	cachedNumberOfSubnets = [mydoc getNumberOfSubnets];
	
	// some counter and helper vars:
	
	// stores the current row we work on.
	unsigned int row=1;
	
	// stores the last network we worked on.
	unsigned int last_net=0;
	unsigned int subnet_24 = (~0)<<8;

	// a neat litte loop-counter
	unsigned int i;
	
	// lets iterate the documents data and prepare the drawing.
	for (i=0;i<cachedNumberOfSubnets;i++) {

		// retrieve subnet data.
		Subnet* subnet = [mydoc getSubnetAtIndex:i];
    unsigned int subnet_ip = [subnet getIP];		
		
		// prepare a subnet mask (a classic C-class subnet) to isolate the first octet of the ip.
		unsigned char first_oct = (subnet_ip-(subnet_ip&subnet_24));

		// if we are leaving the bounds of the current cidr24 subnet, increase row count.
		if ((subnet_ip&subnet_24)>last_net) {
			// lets count our sins
			unsigned int j;

			// we have to precache the cidr24 nets for the legend, so lets iterate...
			for (j=0;j<[subnet getNumberOfCIDR24Blocks];j++)
			{
				last_net=(subnet_ip&subnet_24);
				[cachedNets addObject:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedInt:last_net]]];
				subnet_ip=(subnet_ip+256);
			};

			row+=[subnet getNumberOfCIDR24Blocks];
			
		};
				
		// if this subnet is only part of the first block...
		if ((first_oct<128)&[[subnet getSize] unsignedIntValue]<=128) {			
			
			// ... draw it only in the first block. 
			SubnetRect *newRect = [SubnetRect SubnetRect];
			[newRect setX:[NSNumber numberWithFloat:(page_x_offset+(ip_multiplier*first_oct))]];
			[newRect setY:[NSNumber numberWithFloat:page_y_offset+(row-[subnet getNumberOfCIDR24Blocks])*row_height]];
			[newRect setWidth:[NSNumber numberWithFloat:[[subnet getSize] floatValue]*ip_multiplier]];
			[newRect setHeight:[NSNumber numberWithFloat:row_height]];
			[newRect setSubnet:subnet];
			
			// away with the rectangle data into the cache.
			[cachedRectangles addObject:newRect];
			
		}

		// if this subnet is only part of the second block...
		if ((first_oct>=128)&[[subnet getSize] unsignedIntValue]<=128) {
			
			// ... draw it only in the second block. 
			SubnetRect *newRect = [SubnetRect SubnetRect];
			[newRect setX:[NSNumber numberWithFloat:(page_x_offset+(ip_multiplier*(first_oct-128)))]];
			[newRect setY:[NSNumber numberWithFloat:page_y_offset+(cachedNumberOfRows*row_height)+block_y_offset + (((row-1)-([subnet getNumberOfCIDR24Blocks])) * row_height)]];
			[newRect setWidth:[NSNumber numberWithFloat:[[subnet getSize] floatValue]*ip_multiplier]];
			[newRect setHeight:[NSNumber numberWithFloat:row_height]];
			[newRect setSubnet:subnet];
			
			// away with the rectangle data into the cache.
			[cachedRectangles addObject:newRect];
			
		}

		// now if its part of both...
		if ((first_oct<128)&[[subnet getSize] unsignedIntValue]>128) {
			
			// ... split it up. draw the first half...
			SubnetRect *newRect = [SubnetRect SubnetRect];
			[newRect setX:[NSNumber numberWithFloat:(page_x_offset+(ip_multiplier*first_oct))]];
			[newRect setY:[NSNumber numberWithFloat:page_y_offset+(row-[subnet getNumberOfCIDR24Blocks])*row_height]];
			[newRect setWidth:[NSNumber numberWithFloat:(128.0)*ip_multiplier]];
			[newRect setHeight:[NSNumber numberWithFloat:(row_height*[subnet getNumberOfCIDR24Blocks])]];
			[newRect setSubnet:subnet];

			// ...then the second.
			SubnetRect *newRect2 = [SubnetRect SubnetRect];
			[newRect2 setX:[NSNumber numberWithFloat:(page_x_offset)]];
			[newRect2 setY:[NSNumber numberWithFloat:page_y_offset+(cachedNumberOfRows*row_height)+block_y_offset + (((row-1)-([subnet getNumberOfCIDR24Blocks])) * row_height)]];
			[newRect2 setWidth:[NSNumber numberWithFloat:(128.0)*ip_multiplier]];
			[newRect2 setHeight:[NSNumber numberWithFloat:(row_height*[subnet getNumberOfCIDR24Blocks])]];
			[newRect2 setSubnet:subnet];
						
			// Note: the calculations above are not really neccessary, for the only net that could span the boundary 
			// of the if clause above is a /24 subnet. the data would be fixed, as there is no mapping of bigger subnets
			// in this program and smaller ones would not hit this code... But maybe, just maybe i reuse this code in 
			// some day for another version of ip... may be then... ah crap. forget it. :) I just shot over the damned target
			// and now I am too lazy to touch this sweet block of code. Hey, just bringing those macbook-processors to burn... 
			
			// away with the rectangle data into the cache.
			[cachedRectangles addObject:newRect];
			[cachedRectangles addObject:newRect2];
			
		}		
				
	};
	
	// as we have a completely new cache, it is time to trigger a redraw.
	[self setNeedsDisplay:YES];
	
};

- (void) displayDrawer:(Subnet*)selSubnet {

	if (selSubnet!=nil) {
		
		[_drawer_subnet_label_ip setStringValue:[NSString stringWithFormat:@"%s/%u",[[selSubnet getIPString] UTF8String],[selSubnet getCIDR]]];
		[_drawer_subnet_label_netmask setStringValue:[selSubnet getNMString]];
		[_drawer_subnet_label_wildcard setStringValue:[selSubnet getWildcardString]];
		[_drawer_subnet_label_usablesize setIntValue:[[selSubnet getSize] intValue]-2];
		[_drawer_subnet_label_firstip setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:[selSubnet getIP]+1]]];
		[_drawer_subnet_label_lastip setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:[selSubnet getBroadcast]-1]]];
		[_drawer_subnet_label_broadcast setStringValue:[selSubnet getBroadcastString]];
		[_drawer_subnet_label_identifier setStringValue:[selSubnet getIdentifier]];
		[_drawer_subnet_label_description setStringValue:[selSubnet getDescription]];

	}
	else
	{
		[_drawer_subnet_label_ip setStringValue:@"n/a"];
		[_drawer_subnet_label_netmask setStringValue:@"n/a"];
		[_drawer_subnet_label_wildcard setStringValue:@"n/a"];
		[_drawer_subnet_label_usablesize setStringValue:@"n/a"];
		[_drawer_subnet_label_firstip setStringValue:@"n/a"];
		[_drawer_subnet_label_lastip setStringValue:@"n/a"];
		[_drawer_subnet_label_broadcast setStringValue:@"n/a"];
		[_drawer_subnet_label_identifier setStringValue:@"n/a"];
		[_drawer_subnet_label_description setStringValue:@"n/a"];
	};
	
	
};

- (IBAction) clearMarkings:(id)sender {
	showHosts=NO;
	[self setNeedsDisplay:YES];	
};

- (void)mouseDown:(NSEvent *)theEvent {
	
	NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	
	unsigned int i;
	BOOL foundOne=NO;

	MyDocument* mydoc = (MyDocument*)_document;
	
	// If there were more clicks than one, start editor.
	if ([theEvent clickCount]>1) [mydoc editSubnet:self];

	for (i=0;i<cachedNumberOfSubnets;i++) { [[mydoc getSubnetAtIndex:i] setSelected:NO]; };
	[self setSelectedSubnet:nil];
	
	[_subnet_table deselectAll:self];

	for (i=0;i<[cachedRectangles count];i++) {
		SubnetRect *tempRect=[cachedRectangles objectAtIndex:i];
				
		if ((
				 ([[tempRect getX] floatValue] < curPoint.x)&
				 ([[tempRect getY] floatValue] < curPoint.y)&
				 (([[tempRect getWidth] floatValue]+[[tempRect getX] floatValue]) > curPoint.x)&
				 (([[tempRect getHeight] floatValue]+[[tempRect getY] floatValue]) > curPoint.y)))
		{
			
			[tempRect setSelected:YES];
			
			Subnet *selSubnet=[tempRect getSubnet];
			[self setSelectedSubnet:selSubnet];
			[_subnet_table selectRowIndexes:[NSIndexSet indexSetWithIndex:[[mydoc subnets] indexOfObject:selSubnet]] byExtendingSelection:FALSE];
			foundOne=YES;
			i=[cachedRectangles count];
				
			[self displayDrawer:selSubnet];
			
			//[selSubnet dump2Log];
			
		};
		
	};	
	
	/*
	 
	 The following code was to check if a mouseclick occured in the listing of subnets below the
	 map itself. This is now obsolete, as the list has been exchanged with a version deployed in a 
	 NSTableView and will only be used for printouts - which ave normally no direct contact with any 
	 kind of mouse pointer...
	
	
	// Now check, if the user clicked one of the items in the network list. To achieve that, iterate all networks.
	for (i=0;i<cachedNumberOfSubnets;i++) {

		// check if the currently chosen subnet is clicked.
		if((curPoint.y<(page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+6)*row_height) + (i * row_height) + row_height))&
			 (curPoint.y>(page_y_offset + block_y_offset + (((cachedNumberOfRows*2)+6)*row_height) + (i * row_height)))) {
			
			// get subnet reference
			Subnet *selSubnet = [mydoc getSubnetAtIndex:i];
			
			// select subnet
			[selSubnet setSelected:TRUE];
			[self setSelectedSubnet:selSubnet];
			[_subnet_table selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:FALSE];
			NSLog(@"%d",i);
			foundOne=TRUE;

			// break loop
			i=cachedNumberOfSubnets;
			
			// display data of subnet in drawer
			[self displayDrawer:selSubnet];
			
			// dump subnet data for debugging
			//[selSubnet dump2Log];
			
		};
		
	};

	*/
	 
	if (!foundOne) {
		[self displayDrawer:nil];
	};
		
	[self setNeedsDisplay:YES];

}

/*- (void)mouseDragged:(NSEvent *)theEvent {

	NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	NSLog(@"x:%f y:%f",curPoint.x,curPoint.y);

};

- (void)mouseMoved:(NSEvent *)theEvent {
	NSPoint curPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	NSLog(@"x:%f y:%f",curPoint.x,curPoint.y);
};

- (void)mouseUp:(NSEvent *)theEvent {

}*/

- (void) showHost:(NSString*)host {
	
	int i=0;
	struct hostent *momipdata = gethostbyname([host UTF8String]);
	
	NSMutableArray *momArray= [[NSMutableArray array] retain];
	
	if (momipdata==NULL) {
		[(MyDocument*)_document displayError:@"The Host you specified could not be resolved." info:@"This usually happens if you enter a DNS Hostname which does not exist, please double check the name and try again."];
		return;
	};
	
	char *currip=momipdata->h_addr_list[i];
	while (currip!=NULL) {
		[momArray addObject:[Subnet convertDotted2ULInt:[NSNumber numberWithUnsignedChar:currip[0]] oct2:[NSNumber numberWithUnsignedChar:currip[1]] oct3:[NSNumber numberWithUnsignedChar:currip[2]] oct4:[NSNumber numberWithUnsignedChar:currip[3]]]];
		i++;
		currip=momipdata->h_addr_list[i];
	};
	
	showHosts=YES;
	[self setFindHostList:momArray];
	
	[self setNeedsDisplay:YES];

};

- (NSArray*) getFindHostList {
	return findHostList;
};

- (void) setFindHostList:(NSArray*)list {
	
	if(findHostList!=list) {
		
		[findHostList release];
		findHostList=[list retain];
		
	};
};

- (Subnet*) getSelectedSubnet {
	return _selected_subnet;
};

- (void) setSelectedSubnet:(Subnet*)subnet {
	
	if(_selected_subnet!=subnet) {
		
		[_selected_subnet release];
		_selected_subnet=[subnet retain];
		
	};
};

- (void) setCachedNumberOfSubnets:(unsigned int)number {
	cachedNumberOfSubnets=number;
};

- (unsigned int) cachedSubnets {
	return cachedNumberOfSubnets;
};

- (void) dealloc {

	[cachedRectangles removeAllObjects];
	[cachedLines removeAllObjects];
	[cachedNets removeAllObjects];

	[cachedRectangles release];
	[cachedLines release];
	[cachedNets release];
	
	[self setSelectedSubnet:nil];

	[notesIcon release];
	
	[super dealloc];
	
};

@end
