//
//  MyDocument.m
//  SubnetMapper
//
//  Created by Thomas Thiel on 16.06.08.
//  Copyright Thomas Thiel 2008 . All rights reserved.
//
// THIS CODE REPRESENTS FREE SOFTWARE AND IS RELEASED 
// UNDER THE GPL LICENSE VERSIN 2. PLEASE RESPECT
// THE TERMS OF THIS LICENSE, WHICH CAN BE REVIEWED AT:
// http://www.gnu.org/licenses/gpl-2.0.html

#import "Subnet.h"
#import "MyDocument.h"
#import "MapView.h"

@implementation MyDocument

- (id) init {
    self = [super init];
    if (self) {
    
			_subnets	= [[NSMutableArray array] retain];
			_add_subnet_panel = nil;
			_edited_subnet=nil;

			notesIcon = [[NSImage imageNamed:@"NSInfo"] retain ];
			[notesIcon setSize:NSMakeSize(12,12)];
			
			[self addSubnet2Doc:[Subnet subnetWithIP:[Subnet convertDotted2ULInt:[NSNumber numberWithUnsignedLong:10] 
																																									 oct2:[NSNumber numberWithUnsignedLong:42] 
																																									 oct3:[NSNumber numberWithUnsignedLong:1] 
																																									 oct4:[NSNumber numberWithUnsignedLong:0]] 
																							cidr:[NSNumber numberWithUnsignedLong:24]]];


		}
    return self;
}

- (NSString *) windowNibName {
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"MyDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *)aController {
    [super windowControllerDidLoadNib:aController];
    // Add any code here that needs to be executed once the windowController has loaded the document's window.
}

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError {

	return [NSKeyedArchiver archivedDataWithRootObject:_subnets];
	
}

- (BOOL) readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError {
	
	NSMutableArray* readSubnets  = [NSKeyedUnarchiver unarchiveObjectWithData:data];
	
	[readSubnets sortUsingSelector:@selector(compare:)];
	[self setSubnets:readSubnets];

	[(MapView*)_mapview refreshCache];

	return YES;

}

- (NSPrintOperation *) printOperationWithSettings:(NSDictionary *)printSettings error:(NSError **)outError {

	NSPrintInfo *print_info = [self printInfo];
	[print_info setHorizontalPagination:NSFitPagination];
	
	return [NSPrintOperation printOperationWithView:_mapview printInfo:print_info];
	
};

- (IBAction) addSubnet:(id)sender {

	if (!_add_subnet_panel)
		//Check the myCustomSheet instance variable to make sure the custom sheet does not already exist.
		[NSBundle loadNibNamed: @"add_subnet" owner: self];
	
	[NSApp beginSheet: _add_subnet_panel
		 modalForWindow: [self windowForSheet]
			modalDelegate: self
		 didEndSelector: @selector(didEndSheet:returnCode:contextInfo:)
				contextInfo: nil];
		
};

- (IBAction) closeEditSubnetSheet: (id)sender {

	int i=0;
	BOOL isOK=TRUE;

	// The User pressed "Save Changes"
	
	if ([sender tag]==1)
	{															       
		
		if (_edited_subnet==nil) {
			[self displayError:@"ERROR: no reference to edited subnet set." info:@"This error is actually very critical. Please send a bug report including all helpful information to the author of this program."];
			return;
		};

		unsigned long int nm = (~0)<<(32-[_edit_subnet_cidr intValue]);

		[_edited_subnet setIdentifier:[_edit_subnet_identifier stringValue]];
		[_edited_subnet setDescription:[_edit_subnet_description stringValue]];
		[_edited_subnet setIP:[Subnet convertDotted2ULInt:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct0 intValue]] 
																						oct2:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct1 intValue]] 
																						oct3:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct2 intValue]] 
																						oct4:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct3 intValue]]]];
		
		[_edited_subnet setNM:[NSNumber numberWithUnsignedInt:nm]];
		[_edited_subnet normalize];
		
		[_edited_subnet setNotes:[_edit_subnet_notes textStorage]];
				
		for (i=0;i<[_subnets count];i++) {
			if (([Subnet is:_edited_subnet overlapping:[_subnets objectAtIndex:i]])&(_edited_subnet!=[_subnets objectAtIndex:i])) isOK=FALSE;
		};

		if (isOK) {
			
			_edited_subnet=nil;
		
			[_subnets sortUsingSelector:@selector(compare:)];
			[(MapView*)_mapview refreshCache];
			[NSApp endSheet:_edit_subnet_panel];
			return;
			
		} else {
			
			[self displayError:@"Unable to add Subnet!" info:@"The data you have entered overlaps an existing subnet. Check your input and try again."];
			
		};

	};
		
	if ([sender tag]==312)
	{									

		_edited_subnet=nil;
		
		[NSApp endSheet:_edit_subnet_panel];

	};
		
};

- (IBAction) closeAddSubnetSheet: (id)sender {

	if ([sender tag])
	{
		// The User pressed ADD
		
		Subnet* newsubnet=nil;
		newsubnet= [Subnet subnetWithIP:[Subnet convertDotted2ULInt:[NSNumber numberWithUnsignedLong:[_add_subnet_ipoct0 intValue]] 
																																		oct2:[NSNumber numberWithUnsignedLong:[_add_subnet_ipoct1 intValue]] 
																																		oct3:[NSNumber numberWithUnsignedLong:[_add_subnet_ipoct2 intValue]] 
																																		oct4:[NSNumber numberWithUnsignedLong:[_add_subnet_ipoct3 intValue]]] 
																																		cidr:[NSNumber numberWithUnsignedLong:[_add_subnet_cidr intValue]]];		
				
		[newsubnet setDescription:[_add_subnet_description stringValue]];
		[newsubnet setIdentifier:[_add_subnet_identifier stringValue]];

		[newsubnet setNotes:[_add_subnet_notes textStorage]];

		
		// set currently selected color (if a Color Panel already exists)
		if ([NSColorPanel sharedColorPanelExists]) [newsubnet setColor:[[[NSColorPanel sharedColorPanel] color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"]];

		[self addSubnet2Doc:newsubnet];
		
	};
	
	[NSApp endSheet:_add_subnet_panel];

};

- (void) didEndSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	
	[sheet orderOut:self];
	
};

- (IBAction) removeSubnet:(id)sender {

	// Yes, we can do that. looks ugly, but as we use a setter function in Subnet which releases after use and we use 
	// an NSAray which releases after removal, too, there is a full featured memory management here.
	// No leak around here, bro!
	[_subnets removeObject:[(MapView*)_mapview getSelectedSubnet]];
	[(MapView*)_mapview scrollPoint:NSMakePoint(0,0)];
	[(MapView*)_mapview refreshCache];

};

- (IBAction) showColorPanel:(id)sender {

	[[NSColorPanel sharedColorPanel] orderFront:nil];
	
};

- (void) changeColor:(id)sender {

	[[(MapView*)_mapview getSelectedSubnet] setColor:[[sender color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"]];
	[_mapview setNeedsDisplay:YES];

};

- (IBAction) editAllSubnets:(id)sender {
	NSLog(@"editAllSubnets called [UNUSED] (self:%p,sender:%p)",self,sender);
};

- (IBAction) searchHost:(id)sender{
	if	([[sender stringValue]length]>1) [(MapView*)_mapview showHost:[sender stringValue]];
	else 
	{
		[sender insertText:@""];
		[(MapView*)_mapview clearMarkings:sender];
	};
};

- (void) editSingleSubnet:(Subnet*)subnet {

	if (!_edit_subnet_panel)
		//Check the panel instance variable to make sure the custom panel does not already exist.
		[NSBundle loadNibNamed: @"edit_subnet" owner: self];

	_edited_subnet=subnet;
	
	unsigned long int ip = [_edited_subnet getIP];
	unsigned char ip_oct0 = *((unsigned char*)(&ip)+3);
	unsigned char ip_oct1 = *((unsigned char*)(&ip)+2);
	unsigned char ip_oct2 = *((unsigned char*)(&ip)+1);
	unsigned char ip_oct3 = *((unsigned char*)(&ip));

	unsigned long int nm = [_edited_subnet getNM];
	unsigned char nm_oct0 = *((unsigned char*)(&nm)+3);
	unsigned char nm_oct1 = *((unsigned char*)(&nm)+2);
	unsigned char nm_oct2 = *((unsigned char*)(&nm)+1);
	unsigned char nm_oct3 = *((unsigned char*)(&nm));
	
	[_edit_subnet_ipoct0 setIntValue:ip_oct0];
	[_edit_subnet_ipoct1 setIntValue:ip_oct1];
	[_edit_subnet_ipoct2 setIntValue:ip_oct2];
	[_edit_subnet_ipoct3 setIntValue:ip_oct3];

	[_edit_subnet_nmoctet0 setIntValue:nm_oct0];
	[_edit_subnet_nmoctet1 setIntValue:nm_oct1];
	[_edit_subnet_nmoctet2 setIntValue:nm_oct2];
	[_edit_subnet_nmoctet3 setIntValue:nm_oct3];
	
	[_edit_subnet_cidr setIntValue:[_edited_subnet getCIDR]];
	[_edit_subnet_cidr_stepper setIntValue:[_edited_subnet getCIDR]];
	
	[_edit_subnet_cidrdesc setState:NO];
	[_edit_subnet_identifier setEditable:YES];
	
	[_edit_subnet_description setStringValue:[_edited_subnet getDescription]];
	[_edit_subnet_identifier setStringValue:[_edited_subnet getIdentifier]];
			
	[_edit_subnet_label_ip setStringValue:[NSString stringWithFormat:@"%s/%u",[[_edited_subnet getIPString] UTF8String],[_edited_subnet getCIDR]]];
	[_edit_subnet_label_netmask setStringValue:[_edited_subnet getNMString]];
	[_edit_subnet_label_wildcard setStringValue:[_edited_subnet getWildcardString]];
	[_edit_subnet_label_usablesize setIntValue:[[_edited_subnet getSize] unsignedLongValue]-2];
	[_edit_subnet_label_firstip setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedInt:[_edited_subnet getIP]+1]]];
	[_edit_subnet_label_lastip setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedInt:[_edited_subnet getBroadcast]-1]]];
	[_edit_subnet_label_broadcast setStringValue:[_edited_subnet getBroadcastString]];
	[_edit_subnet_label_identifier setStringValue:[_edited_subnet getIdentifier]];
	[_edit_subnet_label_description setStringValue:[_edited_subnet getDescription]];
		
	if ([_edited_subnet notes] != nil ) {
		[[_edit_subnet_notes textStorage] setAttributedString:[_edited_subnet notes]];
	} else [_edit_subnet_notes insertText:@""];
	
	[NSApp beginSheet: _edit_subnet_panel
		 modalForWindow: [self windowForSheet]
			modalDelegate: self
		 didEndSelector: @selector(didEndSheet:returnCode:contextInfo:)
				contextInfo: nil];
		
};

- (IBAction) editSubnet:(id)sender {
	
	[self editSingleSubnet:[(MapView*)_mapview getSelectedSubnet]];

};

- (void) awakeFromNib {
	
	[(MapView*)_mapview refreshCache];
	
};

- (void) displayError:(NSString*)error info:(NSString*)info {
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert addButtonWithTitle:@"OK"];
	[alert setMessageText:error];
	[alert setInformativeText:info];
	[alert setAlertStyle:NSWarningAlertStyle];
	
/*	
	// a sheet looks so much better than a modal box. but if we try to display a error sheet while another sheet is visible, 
	// our error will be suppressed and will never be seen. saaaaaad. Even if we try to kill the existing sheet in advance...
	[self didEndSheet:_add_subnet_panel returnCode:0 contextInfo:nil];
	[alert beginSheetModalForWindow:[self windowForSheet] 
										modalDelegate:self 
									 didEndSelector:@selector(didEndSheet:returnCode:contextInfo:) 
											contextInfo:nil];
*/

	// so we display a modal box, which will be visible even if a sheet is visible.
	[alert runModal];
	
};

- (unsigned int) getNumberOfSubnets {
	return [_subnets count];
};

- (unsigned int) getNumberOfNets {
	
	NSArray *sortedArray = [_subnets sortedArrayUsingSelector:@selector(compare:)];
	
	unsigned int ctr=0;
	unsigned int i=0;
	unsigned int lastIP=0;
	
	// the following is a subnetmask of 255.255.255.0
	unsigned int cidr26=((~0)<<8);

	// iterate all subnets.
	for (i=0;i<[self getNumberOfSubnets];i++) {

		// get the network ip of the ip with cidr /26 and check if it is the same net. if so, increase counter.
		unsigned int momIP=[[sortedArray objectAtIndex:i] getIP];
		if ((momIP&cidr26)>lastIP) { ctr+=[[sortedArray objectAtIndex:i] getNumberOfCIDR24Blocks]; lastIP=momIP&cidr26; } 
		
	};
	
	// return count
	return ctr;
		
};

- (Subnet*) getSubnetAtIndex:(unsigned int)index {
	return [_subnets objectAtIndex:index];
};

- (void) addSubnet2Doc:(Subnet*)subnet {
	
	int i=0;
	BOOL isOK=TRUE;
	
	for (i=0;i<[_subnets count];i++) {
		if ([Subnet is:subnet overlapping:[_subnets objectAtIndex:i]]) isOK=FALSE;
	};
	
	if (isOK) {
	
		[_subnets addObject:subnet];
		[_subnets sortUsingSelector:@selector(compare:)];
		
	} else {
		
		[self displayError:@"Unable to add Subnet!" info:@"The data you have entered overlaps an existing subnet. Check your input and try again."];
		
	};

	[(MapView*)_mapview refreshCache];
		
};

- (void) setSubnets:(NSMutableArray*)subnets {

	if((_subnets!=subnets)&(subnets!=nil)) {
		
		[_subnets release];
		_subnets=[subnets retain];
		
	};

};

- (NSMutableArray*) subnets {

	return _subnets;

};

- (IBAction) checkEditSubnetSheetInput:(id)sender {
	
	unsigned long int cidr=0;
	unsigned long int nm=0;
		
	switch([sender tag])
	{
		case 1:
			// The CIDR List
			cidr=[sender intValue];
			[_edit_subnet_cidr setIntValue:cidr];
			nm=((~0)<<(32-cidr));
			[_edit_subnet_nmoctet0 setIntValue:((unsigned char)*((unsigned char*)(((void*)&nm)+3)))];
			[_edit_subnet_nmoctet1 setIntValue:((unsigned char)*((unsigned char*)(((void*)&nm)+2)))];
			[_edit_subnet_nmoctet2 setIntValue:((unsigned char)*((unsigned char*)(((void*)&nm)+1)))];
			[_edit_subnet_nmoctet3 setIntValue:((unsigned char)*((unsigned char*)((void*)&nm)))];
			if ([_edit_subnet_cidrdesc state]==NSOnState) [_edit_subnet_identifier setStringValue:[NSString stringWithFormat:@"%d.%d.%d.%d/%d",[_edit_subnet_ipoct0 intValue],[_edit_subnet_ipoct1 intValue],[_edit_subnet_ipoct2 intValue],[_edit_subnet_ipoct3 intValue],[_edit_subnet_cidr intValue]]];
			break;
			case 2:
			// The last subnet mask octet
			break;	
			case 3: 
			// First IP octet
			case 4: 
			// Second IP octet
			case 5:
			// Third IP octet
			case 6: 
			// Forth IP octet
			if ([sender intValue]>255) [sender setStringValue:[NSString stringWithFormat:@"%d",255]];
			if ([sender intValue]<0) [sender setStringValue:[NSString stringWithFormat:@"%d",0]];
			if ([_edit_subnet_cidrdesc state]==NSOnState) [_edit_subnet_identifier setStringValue:[NSString stringWithFormat:@"%d.%d.%d.%d/%d",[_edit_subnet_ipoct0 intValue],[_edit_subnet_ipoct1 intValue],[_edit_subnet_ipoct2 intValue],[_edit_subnet_ipoct3 intValue],[_edit_subnet_cidr intValue]]];
			break;
			case 10:
			// Identifier
			break;
			case 11:
			// Description
			break;
			case 20:
			// checkbox CIDR-Notation Description
			if ([sender state]==NSOnState) {
				[_edit_subnet_identifier setEditable:NO];
				[_edit_subnet_identifier setStringValue:[NSString stringWithFormat:@"%d.%d.%d.%d/%d",[_edit_subnet_ipoct0 intValue],[_edit_subnet_ipoct1 intValue],[_edit_subnet_ipoct2 intValue],[_edit_subnet_ipoct3 intValue],[_edit_subnet_cidr intValue]]];
			} else {
				[_edit_subnet_identifier setEditable:YES];
			};
			break;
	};

	unsigned long mom_ip = [[Subnet convertDotted2ULInt:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct0 intValue]] 
																								oct2:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct1 intValue]] 
																								oct3:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct2 intValue]] 
																								oct4:[NSNumber numberWithUnsignedLong:[_edit_subnet_ipoct3 intValue]]] unsignedLongValue];
	
	unsigned long mom_nm = [[Subnet convertDotted2ULInt:[NSNumber numberWithUnsignedInt:[_edit_subnet_nmoctet0 intValue]] 
																								oct2:[NSNumber numberWithUnsignedInt:[_edit_subnet_nmoctet1 intValue]] 
																								oct3:[NSNumber numberWithUnsignedInt:[_edit_subnet_nmoctet2 intValue]] 
																								oct4:[NSNumber numberWithUnsignedLong:[_edit_subnet_nmoctet3 intValue]]] unsignedLongValue];

	unsigned long mom_wc = ~mom_nm;
	unsigned long mom_bc = (mom_ip|mom_wc);
	unsigned long mom_firstip = mom_ip+1;
	unsigned long mom_lastip = mom_bc-1;
	
	
	
	[_edit_subnet_label_ip setStringValue:[NSString stringWithFormat:@"%s/%u",[[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:mom_ip]] UTF8String],[_edit_subnet_cidr intValue]]];
	[_edit_subnet_label_netmask setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:mom_nm]]];
	[_edit_subnet_label_wildcard setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:mom_wc]]];
	[_edit_subnet_label_usablesize setIntValue:mom_wc-1];
	[_edit_subnet_label_firstip setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:mom_firstip]]];
	[_edit_subnet_label_lastip setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:mom_lastip]]];
	[_edit_subnet_label_broadcast setStringValue:[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:mom_bc]]];
	[_edit_subnet_label_identifier setStringValue:[_edit_subnet_identifier stringValue]];
	[_edit_subnet_label_description setStringValue:[_edit_subnet_description stringValue]];
		
};

- (IBAction) checkAddSubnetSheetInput:(id)sender {
	
	unsigned int cidr=0;
	unsigned long int nm=0;
		
	switch([sender tag])
	{
		case 1:
			// The CIDR List
			cidr=[sender intValue];
			[_add_subnet_cidr setIntValue:cidr];
			nm=((~0)<<(32-cidr));
			[_add_subnet_nmoctet0 setIntValue:((unsigned char)*((unsigned char*)(((void*)&nm)+3)))];
			[_add_subnet_nmoctet1 setIntValue:((unsigned char)*((unsigned char*)(((void*)&nm)+2)))];
			[_add_subnet_nmoctet2 setIntValue:((unsigned char)*((unsigned char*)(((void*)&nm)+1)))];
			[_add_subnet_nmoctet3 setIntValue:((unsigned char)*((unsigned char*)((void*)&nm)))];
			if ([_add_subnet_cidrdesc state]==NSOnState) [_add_subnet_identifier setStringValue:[NSString stringWithFormat:@"%d.%d.%d.%d/%d",[_add_subnet_ipoct0 intValue],[_add_subnet_ipoct1 intValue],[_add_subnet_ipoct2 intValue],[_add_subnet_ipoct3 intValue],[_add_subnet_cidr intValue]]];
			break;
		case 2:
			// The last subnet mask octet
			break;	
	  case 3: 
			// First IP octet
		case 4: 
			// Second IP octet
		case 5:
			// Third IP octet
		case 6: 
			// Forth IP octet
			if ([sender intValue]>255) [sender setStringValue:[NSString stringWithFormat:@"%d",255]];
			if ([sender intValue]<0) [sender setStringValue:[NSString stringWithFormat:@"%d",0]];
			if ([_add_subnet_cidrdesc state]==NSOnState) [_add_subnet_identifier setStringValue:[NSString stringWithFormat:@"%d.%d.%d.%d/%d",[_add_subnet_ipoct0 intValue],[_add_subnet_ipoct1 intValue],[_add_subnet_ipoct2 intValue],[_add_subnet_ipoct3 intValue],[_add_subnet_cidr intValue]]];
			break;
		case 10:
			// Identifier
			break;
		case 11:
			// Description
			break;
		case 20:
			// checkbox CIDR-Notation Description
			if ([sender state]==NSOnState) {
				[_add_subnet_identifier setEditable:NO];
				[_add_subnet_identifier setStringValue:[NSString stringWithFormat:@"%d.%d.%d.%d/%d",[_add_subnet_ipoct0 intValue],[_add_subnet_ipoct1 intValue],[_add_subnet_ipoct2 intValue],[_add_subnet_ipoct3 intValue],[_add_subnet_cidr intValue]]];
			} else {
				[_add_subnet_identifier setEditable:YES];
			};
			break;
	};
	
};

- (BOOL)tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex {

	int i=0;
	
	Subnet *selSubnet = [self getSubnetAtIndex:rowIndex];
	
	for (i=0;i<[self getNumberOfSubnets];i++) { [[self getSubnetAtIndex:i] setSelected:NO]; };
	[(MapView*)_mapview setSelectedSubnet:nil];

	[selSubnet setSelected:YES];
	[(MapView*)_mapview setSelectedSubnet:selSubnet];
	
	[(MapView*)_mapview refreshCache];
	[(MapView*)_mapview displayDrawer:selSubnet];
	
	return YES;

};

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex {

	Subnet *subnet=[self getSubnetAtIndex:rowIndex];
	
	if ([[aTableColumn identifier] isEqualToString:@"ID"]) {return [subnet getIdentifier];};
	if ([[aTableColumn identifier] isEqualToString:@"IP"]) {return ([NSString stringWithFormat:@"%s/%u",[[Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:[subnet getIP]]] UTF8String],[subnet getCIDR]]);};
	if ([[aTableColumn identifier] isEqualToString:@"size"]) {return [subnet getSize];};
	if ([[aTableColumn identifier] isEqualToString:@"broadcast"]) {return [subnet getBroadcastString];};
	if ([[aTableColumn identifier] isEqualToString:@"description"]) {return [subnet getDescription];};
	if ([[aTableColumn identifier] isEqualToString:@"notes"]) {
		if (([subnet notes]!=nil)&
				(![[[subnet notes] string] isEqualToString:@""])) {
			return notesIcon;
		};
	};
	return nil;

};

- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex {

	Subnet *subnet=[self getSubnetAtIndex:rowIndex];
	
	if ([[aTableColumn identifier] isEqualToString:@"ID"]) { 
		[subnet setIdentifier:anObject];
		[(MapView*)_mapview refreshCache];
		[(MapView*)_mapview displayDrawer:subnet];
		return;
	};

	if ([[aTableColumn identifier] isEqualToString:@"description"]) { 
		[subnet setDescription:anObject];
		[(MapView*)_mapview displayDrawer:subnet];
		[(MapView*)_mapview refreshCache];
		return;
	};
	
	
};

- (int)numberOfRowsInTableView:(NSTableView *)aTableView {

	return [self getNumberOfSubnets];

};

- (void) dealloc {
	[_subnets removeAllObjects];
	[_subnets release];
	
	[super dealloc];
};

@end
