//
//  Subnet.h
//  SubnetMapper
//
//  Created by Thomas Thiel on 16.06.08.
//  Copyright 2008 Thomas Thiel. All rights reserved.
//
// THIS CODE REPRESENTS FREE SOFTWARE AND IS RELEASED 
// UNDER THE GPL LICENSE VERSIN 2. PLEASE RESPECT
// THE TERMS OF THIS LICENSE, WHICH CAN BE REVIEWED AT:
// http://www.gnu.org/licenses/gpl-2.0.html



#import <Cocoa/Cocoa.h>


@interface Subnet : NSObject <NSCoding> {

	NSNumber *_ipaddress_int;
	NSNumber *_netmask_int;
	
	NSString *_identifier;
	NSString *_description;
	
	NSColor *_color;
	
	NSAttributedString *_notes;
	
	BOOL _selected;
	
}

+ (NSString*) convertULInt2Dotted:(NSNumber*)number;
+ (BOOL) is:(Subnet*)subnet1 overlapping:(Subnet*)subnet2; 
+ (NSNumber*) convertDotted2ULInt:(NSNumber*)oct1 oct2:(NSNumber*)oct2 oct3:(NSNumber*)oct3 oct4:(NSNumber*) oct4;

+ (Subnet*) subnetWithSubnet:(Subnet*)newsubnet;
+ (Subnet*) subnetWithIP:(NSNumber*)newip netmask:(NSNumber*)newnm;
+ (Subnet*) subnetWithIP:(NSNumber*)newip cidr:(NSNumber*)cidr;

- (void)encodeWithCoder:(NSCoder *)coder;
- (id)initWithCoder:(NSCoder *)coder;

- (Subnet*) initWithSubnet:(Subnet*)newsubnet;
- (Subnet*) initWithIP:(NSNumber*)newip netmask:(NSNumber*)newnm;
- (Subnet*) initWithIP:(NSNumber*)newip cidr:(NSNumber*)cidr;

- (BOOL) containsHost:(unsigned long int)host;

- (void) setIP:(NSNumber*)newip;
- (void) setNM:(NSNumber*)newnm;
- (void) setDescription:(NSString*)description;
- (void) setIdentifier:(NSString*)identifier;
- (void) setSelected:(BOOL)state;
- (void) setColor:(NSColor*)newColor;
- (void) setNotes:(NSAttributedString*)newNotes;

- (NSAttributedString*) notes;
- (NSColor*) color;
- (BOOL) isSelected;
- (unsigned long int) getIP;
- (unsigned long int) getLastIP;
- (unsigned long int) getFirstIP;
- (unsigned long int) getNM;
- (unsigned long int) getWildcard;
- (unsigned long int) getBroadcast;
- (unsigned long int) getNumberOfCIDR24Blocks;
 
- (NSString*) getIPString;
- (NSString*) getNMString;
- (NSString*) getBroadcastString;
- (NSString*) getWildcardString;
- (NSString*) getFirstIPString;
- (NSString*) getLastIPString;
- (NSString*) getDescription;
- (NSString*) getIdentifier;

- (NSNumber*) getSize;
- (unsigned char) getCIDR;

- (NSString*) toString;
- (void) dump2Log;

- (void) normalize;
- (NSComparisonResult) compare:(Subnet*)subnet2;

@end
