//
//  Subnet.m
//  SubnetMapper
//
//  Created by Thomas Thiel on 16.06.08.
//  Copyright 2008 Thomas Thiel. All rights reserved.
//
// THIS CODE REPRESENTS FREE SOFTWARE AND IS RELEASED 
// UNDER THE GPL LICENSE VERSIN 2. PLEASE RESPECT
// THE TERMS OF THIS LICENSE, WHICH CAN BE REVIEWED AT:
// http://www.gnu.org/licenses/gpl-2.0.html

#import "Subnet.h"

const unsigned long int std_ip=3232238083;
const unsigned long int std_nm=4294967040;


@implementation Subnet

-(id)init {
	
	self = [self initWithIP:[NSNumber numberWithUnsignedLong:std_ip] netmask:[NSNumber numberWithUnsignedLong:std_nm]];
	return self;
	
};

- (void) setNM:(NSNumber*)newnm {
	
	if(_netmask_int!=newnm) {
		
		[_netmask_int release];
		_netmask_int=[newnm retain];
		
	};
	
};

- (void) setIP:(NSNumber*)newip {
	
	if(_ipaddress_int!=newip) {
		
		[_ipaddress_int release];
		_ipaddress_int=[newip retain];
		
	};
	
};

- (void) setDescription:(NSString*)description {
	
	if((_description!=description)&(description!=nil)) {
		
		[_description release];
		_description=[description retain];
		
	};
};

- (void) setIdentifier:(NSString*)identifier {
	
	if((_identifier!=identifier)&(identifier!=nil)) {
		
		[_identifier release];
		_identifier=[identifier retain];
		
	};
};

- (void) setColor:(NSColor*)newColor {
	
	if(_color!=newColor) {
		
		[_color release];
		_color=[newColor retain];
		
	};
	
};

- (void) setNotes:(NSAttributedString*)text {

	if(_notes!=text) {
		
		[_notes release];
		
		// OK, this one is special. We normally get the Notes from a NSTextVie testStorage, which is a 
		// reference to the NSTextView internal storage. This is mutable, so we have to make sure, we have 
		// a true copy of the value o the text view. otherwise we end up using the same storage for all new 
		// or edited Subnets - which results in changing not only one note but many... ugly. the copy selector 
		// retains the copied object automagically, so we just need to release it in the dealloc method or one 
		// line above, when overwriting the notes attribute.
		
		_notes=[text copy];
		
	};
	
};

- (NSAttributedString*) notes {

	return _notes;
	
};

- (NSColor*) color {
	return _color;
};

- (unsigned long int) getNM {
	
	return [_netmask_int unsignedLongValue];
	
};

- (unsigned long int) getIP {
	
	return [_ipaddress_int unsignedLongValue];
	
};

- (Subnet*) initWithSubnet:(Subnet*)newsubnet {
	
	self = [self initWithIP:[NSNumber numberWithUnsignedLong:[newsubnet getIP]] netmask:[NSNumber numberWithUnsignedLong:[newsubnet getNM]]];
	return self;
	
};

- (Subnet*) initWithIP:(NSNumber*)newip cidr:(NSNumber*)cidr {
	
	// YES, I am an old C Programmer. :P
	unsigned long int cidr_usable = [cidr unsignedLongValue];
	
	if (cidr_usable>32) cidr_usable=32;
	if (cidr_usable<0) cidr_usable=0;
	
	// THIS is a netmask with all the Bits set, (CIDR /32).
	unsigned long int host_netmask = ~0;
	
  // Now we shift the bits we do not need out of our way. Dirty, but QUICK. 
	host_netmask <<= (32-cidr_usable);
	
	// There is one special case:
	if (!cidr_usable) host_netmask=0;
	
	// We can now allocate the object with IP and Netmask, as usual.
	self = [self initWithIP:newip netmask:[NSNumber numberWithUnsignedLong:host_netmask]];
	return self;
};

// std constructor
- (Subnet*) initWithIP:(NSNumber*)newip netmask:(NSNumber*)newnetmask {
	
	self= [super init];
	
	if (self)
	{	
		[self setIP:newip];
		[self setNM:newnetmask];
		//NSLog(@"Hello, I am an IP Subnet object with the IP %s, and the nmask %s.",[[self getIPString] UTF8String],[[self getNMString] UTF8String]);
		[self normalize];
		[self setIdentifier:[NSString stringWithFormat:@"%s/%d",[[self getIPString] UTF8String],[self getCIDR]]];
		[self setDescription:[NSString stringWithFormat:@"created through: %s/%d",[[Subnet convertULInt2Dotted:newip] UTF8String],[self getCIDR]]];
		[self setColor:[NSColor blueColor]];
		[self setNotes:nil];
	};
	
	return self;
	
};

- (NSString*) getIPString {

	return [Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:[self getIP]]];

};

- (NSString*) getNMString {
	
	return [Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:[self getNM]]];
	
}; 

- (void) normalize {
		
	// as we want to specify a subnet here, we actually have to make sure, that the ip entered is the network address,
	// not one of the network's host addresses. Just AND IP and NM like we learned in ancient times at school.  
	[self setIP:[NSNumber numberWithUnsignedLong:([self getIP]&[self getNM])]];
	//NSLog(@"Normalizing Subnet IP to %s/%s.",[[self getIPString] UTF8String],[[self getNMString] UTF8String]);
	
};

- (NSNumber*) getSize {
	
	// To get the total cardinality of the available IP Adresses of a Subnet, we just need to invert the
	// netmask (wildcard-bits) and return that as a number...
	unsigned long int nm = [self getNM];
	unsigned long int wc = (~nm)+1;
	
	return [NSNumber numberWithUnsignedLong:wc];
	
};

+ (NSString*) convertULInt2Dotted:(NSNumber*)number {

	// UUUUAAAAAAAH HERE BE DRAGONS...
	// I'm using pointer aritmethics to calculate the netmask octets...
	
	// get the ip into something we can work on with std c methods like in old times.
	unsigned long int currentnm = [number unsignedLongValue];
	// Now we do the bad stuff. Get the ptr to the 4byte ulong.
	void* ptr = &currentnm;
	// now use this address as an offset to select the subsequent bytes from the long. 
	// Don't forget to use clean casting, or the compiler will get unhappy.
	unsigned char oct1 = *(unsigned char*)(ptr+3);
	unsigned char oct2 = *(unsigned char*)(ptr+2);
	unsigned char oct3 = *(unsigned char*)(ptr+1);
	unsigned char oct4 = *(unsigned char*)(ptr);
	
	return [NSString stringWithFormat:@"%u.%u.%u.%u",oct1,oct2,oct3,oct4];
		
};

+ (BOOL) is:(Subnet*)subnet1 overlapping:(Subnet*)subnet2 {

	// lets make sure our subnets are all valid and stuff. no need for foo stuff here.

	// AH, we moved this stuff to the init func, so no need anymore to do it more often.
	//[subnet1 normalize];
	//[subnet2 normalize];
	
	// time for some C magic here again... so lets get the data into something we can use.
	
	unsigned long int subnet_ip1 = [subnet1 getIP];
	unsigned long int subnet_nm1 = [subnet1 getNM];
	unsigned long int subnet_ip2 = [subnet2 getIP];
	unsigned long int subnet_nm2 = [subnet2 getNM];

	// OK, a subnet is part of another subnet 
	//		if ip1 is part of the subnet ip2/nm2 
	// or 
	//		if ip2 is part of the subnet ip1/nm1.

	
	
	
	if (((subnet_ip1&subnet_nm1)==(subnet_ip2&subnet_nm1))||((subnet_ip1&subnet_nm2)==(subnet_ip2&subnet_nm2))) return TRUE;
	
	return FALSE;
	
}; 

+ (NSNumber*) convertDotted2ULInt:(NSNumber*)oct1 oct2:(NSNumber*)oct2 oct3:(NSNumber*)oct3 oct4:(NSNumber*) oct4 {

	unsigned char _oct1= [oct1 unsignedCharValue]; 
	unsigned char _oct2= [oct2 unsignedCharValue]; 
	unsigned char _oct3= [oct3 unsignedCharValue]; 
	unsigned char _oct4= [oct4 unsignedCharValue]; 

	unsigned long int _quadrupled=0;
	
	void* ptr_target = &_quadrupled;
	unsigned char* temp_val=0;
	
	temp_val = (unsigned char*)ptr_target+3;
	*temp_val = _oct1;
	temp_val = (unsigned char*)ptr_target+2;
	*temp_val = _oct2;
	temp_val = (unsigned char*)ptr_target+1;
	*temp_val = _oct3;
	temp_val = (unsigned char*)ptr_target;
	*temp_val = _oct4;
	
	return [NSNumber numberWithUnsignedLong:_quadrupled];
	
};

- (unsigned long) getFirstIP {
	
	if ([self getCIDR]<31)
		return ([self getIP]+1);
	else
		return 0;
	
};

- (unsigned long) getLastIP {
	
	if ([self getCIDR]<31)
		return [self getBroadcast]-1;
	else
		return 0;
	
};

- (unsigned long) getBroadcast {
	
	return ([self getIP]|[self getWildcard]);
	
};

- (unsigned long) getWildcard {
	
	return (~[self getNM]);
	
};

- (NSString*) getBroadcastString {
	
	return ([Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:([self getBroadcast])]]);
	
};

- (NSString*) getWildcardString {
	
	return ([Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:([self getWildcard])]]);
	
};

- (NSString*) getFirstIPString {
	
	if ([self getCIDR]<31)
		return ([Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:([self getFirstIP])]]);
	else
		return @"-- n/a --";
};

- (NSString*) getLastIPString {
	
	if ([self getCIDR]<31)
		return ([Subnet convertULInt2Dotted:[NSNumber numberWithUnsignedLong:([self getLastIP])]]);
	else
		return @"-- n/a --";
	
};

- (NSString*) getDescription {
	return _description;
};

- (NSString*) getIdentifier {
	return _identifier;
};

- (unsigned char) getCIDR {

	unsigned char cidr=32;
	unsigned long wildcard=~[self getNM];
	
	while (wildcard>0) {
		cidr--;
		wildcard>>=1;
	};

	return cidr;
};

- (void) dump2Log {

	NSLog(@"---DUMP START-------------------------------");
	NSLog(@" Object Reference:       %p",self);
	NSLog(@"--------------------------------------------");
	NSLog(@" Network:                %s/%u",[[self getIPString] UTF8String],[self getCIDR]);
	NSLog(@" Netmask:                %s",[[self getNMString] UTF8String]);
	NSLog(@" Wildcard Bits:          %s",[[self getWildcardString] UTF8String]);
	NSLog(@" Size:                   %u",[[self getSize] unsignedLongValue]);
	NSLog(@" First Usable IP:        %s",[[self getFirstIPString] UTF8String]);
	NSLog(@" Last Usable IP:         %s",[[self getLastIPString] UTF8String]);
	NSLog(@" Broadcast Address:      %s",[[self getBroadcastString] UTF8String]);
	NSLog(@" CIDR24 Blocks:          %u",[self getNumberOfCIDR24Blocks]);
	NSLog(@"--------------------------------------------");
	NSLog(@" Identifier:");
	NSLog(@"   %s",[[self getIdentifier] UTF8String]);
	NSLog(@" Description:");
	NSLog(@"   %s",[[self getDescription] UTF8String]);
	NSLog(@"--------------------------------------------");
	NSLog(@" Notes dump:");
	NSLog(@"   %s",[[[self notes] string] UTF8String]);
	NSLog(@"---------------------------------DUMP END---");

};

+ (Subnet*) subnetWithSubnet:(Subnet*)newsubnet {

	return [[[Subnet alloc] initWithSubnet:newsubnet] autorelease]; 

};

+ (Subnet*) subnetWithIP:(NSNumber*)newip netmask:(NSNumber*)newnm {

	return [[[Subnet alloc] initWithIP:newip netmask:newnm] autorelease]; 
	
};

+ (Subnet*) subnetWithIP:(NSNumber*)newip cidr:(NSNumber*)cidr {

	return [[[Subnet alloc] initWithIP:newip cidr:cidr] autorelease]; 
	
};

- (void) dealloc {

	[self setIP:nil];
	[self setNM:nil];
	[self setDescription:nil];
	[self setIdentifier:nil];
	[self setColor:nil];
	[self setNotes:nil];

	
	[super dealloc];

};

- (NSComparisonResult) compare:(Subnet*)subnet2 {
//	return [[NSNumber numberWithUnsignedInt:[self getIP]] compare:[NSNumber numberWithUnsignedInt:[subnet2 getIP]]];
	unsigned int ip1 = [self getIP];
	unsigned int ip2 = [subnet2 getIP];
	
	if (ip1>ip2) return NSOrderedDescending;
	if (ip1==ip2) return NSOrderedSame;
	if (ip1<ip2) return NSOrderedAscending;

	// will never be reached, as long as math is still there.
	return 0;
};

- (void)encodeWithCoder:(NSCoder *)coder {

  if ( [coder allowsKeyedCoding] ) {
		[coder encodeObject:_ipaddress_int forKey:@"SubnetIP"];
		[coder encodeObject:_netmask_int forKey:@"SubnetNM"];
		[coder encodeObject:_identifier forKey:@"SubnetIdentifier"];
		[coder encodeObject:_description forKey:@"SubnetDescription"];
		[coder encodeObject:_color forKey:@"SubnetColor"];
		[coder encodeObject:_notes forKey:@"SubnetNotes"];
	}
	else
	{
		[coder encodeObject:_ipaddress_int];
		[coder encodeObject:_netmask_int];
		[coder encodeObject:_identifier];
		[coder encodeObject:_description];
		[coder encodeObject:_color];
		[coder encodeObject:_notes];
	};

};

- (id)initWithCoder:(NSCoder *)coder {

	self = [[[Subnet alloc] init] retain] ;
	
  if ( [coder allowsKeyedCoding] ) {
		[self setIP:[coder decodeObjectForKey:@"SubnetIP"]];
		[self setNM:[coder decodeObjectForKey:@"SubnetNM"]];
		[self setIdentifier:[coder decodeObjectForKey:@"SubnetIdentifier"]];
		[self setDescription:[coder decodeObjectForKey:@"SubnetDescription"]];
		[self setColor:[coder decodeObjectForKey:@"SubnetColor"]];
		[self setNotes:[coder decodeObjectForKey:@"SubnetNotes"]];
	} else {
		[self setIP:[coder decodeObject]];
		[self setNM:[coder decodeObject]];
		[self setIdentifier:[coder decodeObject]];
		[self setDescription:[coder decodeObject]];
		[self setColor:[coder decodeObject]];
		[self setNotes:[coder decodeObject]];
	};
	
	// Normalize the read 
	[self normalize];
	
	// make sure that old files do not confuse the edit dialogue, so filter out those unset notes...
	if ([self notes]==nil) [self setNotes:[[NSAttributedString alloc] initWithString:@""]];
	
	return self;
	
};

- (unsigned long int) getNumberOfCIDR24Blocks {
	unsigned long int nm = [self getNM];
	
	// complement the netmask and shift it 8 bits to the right and you get the number of /24
	// nets in the subnet (-1, because the zero counts a a net again).
	unsigned long int numberOfCIDRBlocks=(((~nm)>>8)+1);
	
	return numberOfCIDRBlocks;
};

- (BOOL) containsHost:(unsigned long int)host;
{
	[self normalize];
	
	if ((host&[self getNM])==[self getIP])
		return YES;
	else
		return NO;
};

- (BOOL) isSelected {
	return _selected;
};

- (void) setSelected:(BOOL)state {
	_selected=state;
};

- (NSString*) toString {
	return [NSString stringWithFormat:@"%s \t%s/%u \t%d \t%s \t%s",[[self getIdentifier] UTF8String],[[self getIPString] UTF8String],[self getCIDR],[[self getSize] unsignedLongValue]-2,[[self getBroadcastString] UTF8String],[[self getDescription] UTF8String]];
};

@end
